from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ChunkerConfig")


@_attrs_define
class ChunkerConfig:
    """
    Attributes:
        max_chunk_tokens (int | Unset): Maximum tokens per chunk. Chunks exceeding this are force-split at token
            boundaries. Default: 8000.
    """

    max_chunk_tokens: int | Unset = 8000
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        max_chunk_tokens = self.max_chunk_tokens

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if max_chunk_tokens is not UNSET:
            field_dict["MAX_CHUNK_TOKENS"] = max_chunk_tokens

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        max_chunk_tokens = d.pop("MAX_CHUNK_TOKENS", UNSET)

        chunker_config = cls(
            max_chunk_tokens=max_chunk_tokens,
        )

        chunker_config.additional_properties = d
        return chunker_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
