from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.non_developer_config_agents import NonDeveloperConfigAgents
    from ..models.non_developer_config_artifactllm import NonDeveloperConfigArtifactllm


T = TypeVar("T", bound="NonDeveloperConfig")


@_attrs_define
class NonDeveloperConfig:
    """Limited configuration response for non-developer users

    Attributes:
        agents (NonDeveloperConfigAgents):
        artifact_llm (NonDeveloperConfigArtifactllm):
    """

    agents: NonDeveloperConfigAgents
    artifact_llm: NonDeveloperConfigArtifactllm
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        agents = self.agents.to_dict()

        artifact_llm = self.artifact_llm.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "Agents": agents,
                "ArtifactLLM": artifact_llm,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.non_developer_config_agents import NonDeveloperConfigAgents
        from ..models.non_developer_config_artifactllm import NonDeveloperConfigArtifactllm

        d = dict(src_dict)
        agents = NonDeveloperConfigAgents.from_dict(d.pop("Agents"))

        artifact_llm = NonDeveloperConfigArtifactllm.from_dict(d.pop("ArtifactLLM"))

        non_developer_config = cls(
            agents=agents,
            artifact_llm=artifact_llm,
        )

        non_developer_config.additional_properties = d
        return non_developer_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
